( cd usr/lib ; rm -rf libgamin-1.so.0 )
( cd usr/lib ; ln -sf libgamin-1.so.0.1.5 libgamin-1.so.0 )
( cd usr/lib ; rm -rf libgamin-1.so )
( cd usr/lib ; ln -sf libgamin-1.so.0.1.5 libgamin-1.so )
( cd usr/lib ; rm -rf libfam.so.0 )
( cd usr/lib ; ln -sf libfam.so.0.0.0 libfam.so.0 )
( cd usr/lib ; rm -rf libfam.so )
( cd usr/lib ; ln -sf libfam.so.0.0.0 libfam.so )
